package com.example.myapplication;

import android.Manifest;
import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.view.KeyEvent;
import android.view.SurfaceView;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.example.myapplication.view.WinKeyboardView;
import com.huawei.cloudapp.CvrSDK;
import com.huawei.cloudapp.IntentCVR;
import com.huawei.cloudapp.util.Log;

import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;


/**
 * 抽象类，定义公用的方法
 * */
public class VrMainActivity extends Activity implements ActivityCompat.OnRequestPermissionsResultCallback,
        View.OnClickListener {
    private static final String TAG = "VrMainActivity";
    private static final boolean ENABLE_STATISTICS = false;
    private static final int REQUEST_CODE = 1;
    private static final int UPDATE_HINT = 1000;

    private List<String> mPermissionList = new ArrayList<>();
    public static String[] permissions = {
        Manifest.permission.READ_EXTERNAL_STORAGE,
        Manifest.permission.WRITE_EXTERNAL_STORAGE
    };

    private Handler handler = new Handler() {
        @Override
        public void handleMessage(@NonNull Message msg) {
            switch (msg.what) {
                case UPDATE_HINT:
                    tvHint.setText((String)msg.obj);
                    break;
                default:
                    break;
            }
            super.handleMessage(msg);
        }
    };

    private SurfaceView surfaceView;
    private WinKeyboardView keyboardView;
    private ImageButton ibKeyboard;
    private ImageButton ibPlay;
    private ImageButton ibRestart;
    private TextView tvHint;
    private TextView tvLog;
    private Timer timer;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);// 设置全屏
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        setContentView(R.layout.activity_cloudapp);

        if (isHasPermission()) {
            init();
            CvrSDK.getInstance().onActivityCreate(this, surfaceView);
        } else {
            requestPer();
        }
        Log.d(TAG, "onCreate: finish request permission");
        super.onCreate(savedInstanceState);
    }

    private void init() {
        surfaceView = findViewById(R.id.surfaceview);
        tvHint = findViewById(R.id.tv_ca_hint);
        tvLog = findViewById(R.id.tv_log);
        keyboardView = findViewById(R.id.keyboard_view);
        ibKeyboard = findViewById(R.id.ib_keyboard);
        ibKeyboard.setOnClickListener(this);
        ibPlay = findViewById(R.id.ib_play);
        ibPlay.setOnClickListener(this);
        ibRestart = findViewById(R.id.ib_restart);
        ibRestart.setOnClickListener(this);
        initTimer();
        IntentCVR.setErrCallback(new IntentCVR.CvrErrCallback() {
            @Override
            public void onErrMessage(String s, String s1) {
                //s为简略的错误信息，s1为详细的，当前会反馈的信息包含天空盒的错误信息和获取deviceID的错误信息
                android.util.Log.i(TAG,s + s1);
                Message message = new Message();
                message.what = UPDATE_HINT;
                message.obj = "ErrCode:" + s + " ErrMsg:" + s1;
                handler.sendMessage(message);
            }

            @Override
            public void onTokenOutdated() {
                //当前的token在后台的时间已经超出timelimit，重新建立连接，timelimit默认为60
                android.util.Log.i(TAG, "onTokenOutdated");
            }
        });
    }

    private void initTimer() {
        if (!ENABLE_STATISTICS || timer != null) {
            return;
        }

        timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                tvLog.post(() -> tvLog.setText(CvrSDK.getInstance().getStatistics()));
            }
        }, 1000, 1000);
    }

    @Override
    protected void onResume() {
        super.onResume();
        CvrSDK.getInstance().onActivityResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        CvrSDK.getInstance().onActivityPause();
    }

    @Override
    protected void onStop() {
        super.onStop();
        CvrSDK.getInstance().onActivityStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        CvrSDK.getInstance().onActivityDestroy();
        if (timer != null) {
            timer.cancel();
        }
    }

    //检查是否已经有所需权限
    private boolean isHasPermission() {
        for (int i = 0; i < permissions.length; i++) {
            if (ActivityCompat.checkSelfPermission(this, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }

    //请求缺失权限
    private void requestPer() {
        mPermissionList.clear();
        for (int i = 0; i < permissions.length; i++) {
            if (ActivityCompat.checkSelfPermission(this, permissions[i]) != PackageManager.PERMISSION_GRANTED) {
                mPermissionList.add(permissions[i]);
            }
        }
        if (!mPermissionList.isEmpty()) {
            String[] permissions = mPermissionList.toArray(new String[mPermissionList.size()]);//将List转为数组
            ActivityCompat.requestPermissions(this, permissions, REQUEST_CODE);
        }
    }

    /**
     * 请求文件读写权限的结果
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == REQUEST_CODE) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                init();
                CvrSDK.getInstance().onActivityCreate(this, surfaceView);
            } else {
                finish();
            }
        }
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        CvrSDK.getInstance().handleKeyEvent(event);
        return super.dispatchKeyEvent(event);
    }

    long exitTime = 0;
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if ((System.currentTimeMillis() - exitTime) > 2000) {
                Toast.makeText(this, R.string.toback, Toast.LENGTH_SHORT).show();
                exitTime = System.currentTimeMillis();
            } else {
                moveTaskToBack(true);
            }
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }


    private boolean isPlay = true;
    @Override
    public void onClick(View view) {
        if (view == ibKeyboard) {
            int newStatus = keyboardView.getVisibility() == View.VISIBLE ? View.GONE : View.VISIBLE;
            keyboardView.setVisibility(newStatus);
            if (newStatus == View.VISIBLE) {
                keyboardView.invalidate();
            }
        } else if (view == ibPlay) {
            try {
                if (isPlay) {
                    IntentCVR.pauseCVR();
                } else {
                    IntentCVR.resumeCVR();
                }
                isPlay = !isPlay;
            } catch (Exception e) {
                Log.e(TAG, e.toString(), e);
            }
        } else if (view == ibRestart) {
            IntentCVR.restartCVR();
        }
    }
}
