package com.example.myapplication.view;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.StateListDrawable;
import android.graphics.drawable.shapes.RoundRectShape;
import android.os.Build;
import android.support.annotation.RequiresApi;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.SparseArray;
import android.util.StateSet;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.huawei.cloudapp.CvrSDK;
import com.huawei.cloudapp.IntentCVR;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * 类似windows键盘
 *
 * @author shaoheyi
 */
public class WinKeyboardView extends LinearLayout {

    public interface IWinKeyboardViewListener {
        boolean handleWinKeyboardTouchDown(KeyItemInfo info);

        boolean handleWinKeyboardTouchUp(KeyItemInfo info);
    }

    private static final int[] KEYCODE_LINE3 = new int[]{KeyEvent.KEYCODE_TAB, KeyEvent.KEYCODE_Q, KeyEvent.KEYCODE_W,
        KeyEvent.KEYCODE_E, KeyEvent.KEYCODE_R, KeyEvent.KEYCODE_T, KeyEvent.KEYCODE_Y,
        KeyEvent.KEYCODE_U, KeyEvent.KEYCODE_I, KeyEvent.KEYCODE_O, KeyEvent.KEYCODE_P,
        KeyEvent.KEYCODE_LEFT_BRACKET, KeyEvent.KEYCODE_RIGHT_BRACKET, KeyEvent.KEYCODE_BACKSLASH};
    private static final int[] KEYCODE_LINE4 = new int[]{KeyEvent.KEYCODE_CAPS_LOCK, KeyEvent.KEYCODE_A, KeyEvent.KEYCODE_S,
        KeyEvent.KEYCODE_D, KeyEvent.KEYCODE_F, KeyEvent.KEYCODE_G, KeyEvent.KEYCODE_H,
        KeyEvent.KEYCODE_J, KeyEvent.KEYCODE_K, KeyEvent.KEYCODE_L, KeyEvent.KEYCODE_SEMICOLON,
        KeyEvent.KEYCODE_APOSTROPHE, KeyEvent.KEYCODE_ENTER};
    private static final int[] KEYCODE_LINE5 = new int[]{KeyEvent.KEYCODE_SHIFT_LEFT, KeyEvent.KEYCODE_Z, KeyEvent.KEYCODE_X,
        KeyEvent.KEYCODE_C, KeyEvent.KEYCODE_V, KeyEvent.KEYCODE_B, KeyEvent.KEYCODE_N,
        KeyEvent.KEYCODE_M, KeyEvent.KEYCODE_COMMA, KeyEvent.KEYCODE_PERIOD, KeyEvent.KEYCODE_SLASH,
        KeyEvent.KEYCODE_SHIFT_RIGHT, KeyEvent.KEYCODE_DPAD_UP};
    private static final int[] KEYCODE_LINE6 = new int[]{KeyEvent.KEYCODE_CTRL_LEFT, KeyEvent.KEYCODE_ALT_LEFT, KeyEvent.KEYCODE_SPACE,
        KeyEvent.KEYCODE_ALT_RIGHT, KeyEvent.KEYCODE_CTRL_RIGHT, KeyEvent.KEYCODE_DPAD_LEFT, KeyEvent.KEYCODE_DPAD_RIGHT,
        KeyEvent.KEYCODE_DPAD_DOWN};

    private float scale = 1.0f;
    private SparseArray<String> labelMap = new SparseArray<>();
    private IWinKeyboardViewListener winKeyboardListener;
    private SparseArray<String> keyMapPressing = new SparseArray<>();

    public WinKeyboardView(Context context) {
        super(context);
        initKeyboardDataAndView();
    }

    public WinKeyboardView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initKeyboardDataAndView();
    }

    public WinKeyboardView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initKeyboardDataAndView();
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public WinKeyboardView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        initKeyboardDataAndView();
    }

    private void initKeyboardDataAndView() {
        // 键盘第一行
        List<Integer> keyCodeLine1 = new ArrayList<>();
        keyCodeLine1.add(KeyEvent.KEYCODE_ESCAPE);
        for (int i = KeyEvent.KEYCODE_F1; i <= KeyEvent.KEYCODE_F12; i++) {
            keyCodeLine1.add(i);
        }

        // 键盘第二行
        List<Integer> keyCodeLine2 = new ArrayList<>();
        keyCodeLine2.add(KeyEvent.KEYCODE_GRAVE); // `
        for (int i = KeyEvent.KEYCODE_1; i <= KeyEvent.KEYCODE_9; i++) {
            keyCodeLine2.add(i);
        }
        keyCodeLine2.add(KeyEvent.KEYCODE_0);
        keyCodeLine2.add(KeyEvent.KEYCODE_MINUS);
        keyCodeLine2.add(KeyEvent.KEYCODE_EQUALS);
        keyCodeLine2.add(KeyEvent.KEYCODE_DEL);

        // 键盘显示名称
        initLabelMap();
        // 特殊宽的按键的Weight
        SparseArray<Integer> specialWeight = new SparseArray<>();
        specialWeight.put(KeyEvent.KEYCODE_DEL, 2);
        specialWeight.put(KeyEvent.KEYCODE_ENTER, 2);
        specialWeight.put(KeyEvent.KEYCODE_TAB, 2);
        specialWeight.put(KeyEvent.KEYCODE_CAPS_LOCK, 2);
        specialWeight.put(KeyEvent.KEYCODE_SHIFT_RIGHT, 2);
        specialWeight.put(KeyEvent.KEYCODE_SHIFT_LEFT, 2);
        specialWeight.put(KeyEvent.KEYCODE_SPACE, 7);
        // layouts
        setBackgroundColor(Color.parseColor("#dcdcdc"));
        setOrientation(LinearLayout.VERTICAL);
        setShowDividers(SHOW_DIVIDER_MIDDLE);
        setDividerDrawable(getCustomDividerDrawable(1, 2, Color.TRANSPARENT));
        DisplayMetrics dm = getResources().getDisplayMetrics();
        scale = Math.min(dm.widthPixels * 1.0f / dm.density, dm.heightPixels * 1.0f / dm.density) / 320;
        // line...
        addKeyboardLineView(keyCodeLine1, specialWeight);
        addKeyboardLineView(keyCodeLine2, specialWeight);
        addKeyboardLineView(KEYCODE_LINE3, specialWeight);
        addKeyboardLineView(KEYCODE_LINE4, specialWeight);
        addKeyboardLineView(KEYCODE_LINE5, specialWeight);
        addKeyboardLineView(KEYCODE_LINE6, specialWeight);
    }

    private void initLabelMap() {
        for (int keycode = KeyEvent.KEYCODE_A; keycode <= KeyEvent.KEYCODE_Z; keycode++) {
            labelMap.put(keycode, String.valueOf((char) (keycode - KeyEvent.KEYCODE_A + 'a')));
        }

        // 0~9
        String[] numSpecChar = new String[]{")", "!", "@", "#", "$", "%", "^", "&", "*", "("};
        for (int keyCode = KeyEvent.KEYCODE_0; keyCode <= KeyEvent.KEYCODE_9; keyCode++) {
            int index = keyCode - KeyEvent.KEYCODE_0;
            labelMap.put(keyCode, String.format(Locale.CHINA, "%s\n%d", numSpecChar[index], index));
        }

        // F1~F12
        for (int keycode = KeyEvent.KEYCODE_F1; keycode <= KeyEvent.KEYCODE_F12; keycode++) {
            int index = keycode - KeyEvent.KEYCODE_F1;
            labelMap.put(keycode, String.format(Locale.CHINA, "F%d", index + 1));
        }

        // 各行特殊的
        labelMap.put(KeyEvent.KEYCODE_ESCAPE, "ESC");
        labelMap.put(KeyEvent.KEYCODE_GRAVE, "~\n`"); // -= backspace
        labelMap.put(KeyEvent.KEYCODE_MINUS, "_\n-");
        labelMap.put(KeyEvent.KEYCODE_EQUALS, "+\n=");
        labelMap.put(KeyEvent.KEYCODE_DEL, "<--");
        labelMap.put(KeyEvent.KEYCODE_TAB, "TAB"); // TAB[]Enter
        labelMap.put(KeyEvent.KEYCODE_LEFT_BRACKET, "{\n[");
        labelMap.put(KeyEvent.KEYCODE_RIGHT_BRACKET, "}\n]");
        labelMap.put(KeyEvent.KEYCODE_ENTER, "ENTER");
        labelMap.put(KeyEvent.KEYCODE_CAPS_LOCK, "Caps"); // caps lock;'\
        labelMap.put(KeyEvent.KEYCODE_SEMICOLON, ":\n;");
        labelMap.put(KeyEvent.KEYCODE_APOSTROPHE, "\"\n,");
        labelMap.put(KeyEvent.KEYCODE_BACKSLASH, "|\n\\");
        labelMap.put(KeyEvent.KEYCODE_SHIFT_LEFT, "SHIFT"); // shift,./
        labelMap.put(KeyEvent.KEYCODE_COMMA, "<\n,");
        labelMap.put(KeyEvent.KEYCODE_PERIOD, ">\n.");
        labelMap.put(KeyEvent.KEYCODE_SLASH, "?\n/");
        labelMap.put(KeyEvent.KEYCODE_SHIFT_RIGHT, "SHIFT");
        labelMap.put(KeyEvent.KEYCODE_CTRL_LEFT, "CTRL"); // ctrl
        labelMap.put(KeyEvent.KEYCODE_CTRL_RIGHT, "CTRL");
        labelMap.put(KeyEvent.KEYCODE_ALT_LEFT, "ALT");
        labelMap.put(KeyEvent.KEYCODE_ALT_RIGHT, "ALT");
        labelMap.put(KeyEvent.KEYCODE_SPACE, "SPACE");
        labelMap.put(KeyEvent.KEYCODE_DPAD_UP, "上"); // 上下左右
        labelMap.put(KeyEvent.KEYCODE_DPAD_DOWN, "下");
        labelMap.put(KeyEvent.KEYCODE_DPAD_LEFT, "左");
        labelMap.put(KeyEvent.KEYCODE_DPAD_RIGHT, "右");
    }

    private void addKeyboardLineView(int[] keyCodes, SparseArray<Integer> weightMap) {
        LinearLayout line = generateOneLine();
        for (int i = 0; i < keyCodes.length; i++) {
            int keycode = keyCodes[i];
            addKeyboardItemView(line, keycode, weightMap);
        }
    }

    private void addKeyboardLineView(List<Integer> keyCodes, SparseArray<Integer> weightMap) {
        LinearLayout line = generateOneLine();
        for (int i = 0; i < keyCodes.size(); i++) {
            int keycode = keyCodes.get(i);
            addKeyboardItemView(line, keycode, weightMap);
        }
    }

    private LinearLayout generateOneLine() {
        LinearLayout line = new LinearLayout(getContext());
        // divider
        line.setShowDividers(LinearLayout.SHOW_DIVIDER_MIDDLE);
        ShapeDrawable drawable = getCustomDividerDrawable(2, 1, Color.TRANSPARENT);
        line.setDividerDrawable(drawable);
        // line height
        int minHeight = (int) (scale * 60);
        LayoutParams params = new LayoutParams(LayoutParams.MATCH_PARENT, minHeight);
        addView(line, params);
        return line;
    }

    private ShapeDrawable getCustomDividerDrawable(int width, int height, int solidColor) {
        float radius = 0;
        float[] outerR = new float[]{radius, radius, radius, radius, radius, radius, radius, radius};
        RoundRectShape rr = new RoundRectShape(outerR, null, null);
        ShapeDrawable drawable = new ShapeDrawable(rr);
        drawable.getPaint().setColor(solidColor);
        drawable.getPaint().setStyle(Paint.Style.STROKE);
        drawable.getPaint().setStrokeWidth(2);
        drawable.setIntrinsicWidth(width);
        drawable.setIntrinsicHeight(height);
        return drawable;
    }

    private void addKeyboardItemView(LinearLayout parent, int keycode, SparseArray<Integer> weightMap) {
        StateListDrawable drawable = new StateListDrawable();
        drawable.addState(new int[]{android.R.attr.state_pressed}, new ColorDrawable(Color.parseColor("#1e90ff")));
        drawable.addState(StateSet.WILD_CARD, new ColorDrawable(Color.parseColor("#ebebeb")));

        String label = labelMap.indexOfKey(keycode) >= 0 ? labelMap.get(keycode) : "";
        TextView tvKey = new TextView(getContext());
        tvKey.setText(label);
        tvKey.setGravity(Gravity.CENTER);
        tvKey.setBackground(drawable);
        tvKey.setTextSize(TypedValue.COMPLEX_UNIT_PX, scale * 15);
        tvKey.setOnTouchListener(keyTouchListener);
        tvKey.setTag(new KeyItemInfo(keycode, label));
        tvKey.setClickable(true);
        tvKey.setFocusable(true);
        LayoutParams params = new LayoutParams(0, LayoutParams.MATCH_PARENT);
        params.weight = weightMap.indexOfKey(keycode) >= 0 ? weightMap.get(keycode) : 1;
        parent.addView(tvKey, params);
    }

    public boolean isKeycodePressing(int keyCode) {
        return keyMapPressing.indexOfKey(keyCode) >= 0;
    }

    public boolean isAltPressing() {
        return isKeycodePressing(KeyEvent.KEYCODE_ALT_LEFT) || isKeycodePressing(KeyEvent.KEYCODE_ALT_RIGHT);
    }

    public boolean isShiftPressing() {
        return isKeycodePressing(KeyEvent.KEYCODE_SHIFT_LEFT) || isKeycodePressing(KeyEvent.KEYCODE_SHIFT_RIGHT);
    }

    public boolean isCtrlPressing() {
        return isKeycodePressing(KeyEvent.KEYCODE_CTRL_LEFT) || isKeycodePressing(KeyEvent.KEYCODE_CTRL_RIGHT);
    }

    public IWinKeyboardViewListener getWinKeyboardListener() {
        return winKeyboardListener;
    }

    public void setWinKeyboardListener(IWinKeyboardViewListener winKeyboardListener) {
        this.winKeyboardListener = winKeyboardListener;
    }

    private OnTouchListener keyTouchListener = new OnTouchListener() {
        @Override
        public boolean onTouch(View view, MotionEvent event) {
            KeyItemInfo info = view.getTag() instanceof KeyItemInfo ? (KeyItemInfo) view.getTag() : null;
            if (info == null) {
                return false;
            }

            if (event.getAction() == MotionEvent.ACTION_DOWN) {
                sendPressingKeysToPlayer();

                keyMapPressing.put(info.keycode, null);

                if (IntentCVR.getCaPlayer() != null) {
                    CvrSDK.getInstance().handleKeyCodeDown(info.keycode);
                    return false;
                }

                if (winKeyboardListener != null) {
                    return winKeyboardListener.handleWinKeyboardTouchDown(info);
                }
            } else if (event.getAction() == MotionEvent.ACTION_UP) {
                keyMapPressing.remove(info.keycode);

                if (IntentCVR.getCaPlayer() != null) {
                    CvrSDK.getInstance().handleKeyCodeUp(info.keycode);
                    return false;
                }

                if (winKeyboardListener != null) {
                    return winKeyboardListener.handleWinKeyboardTouchUp(info);
                }
            }
            return false;
        }
    };

    private void sendPressingKeysToPlayer() {
        if (IntentCVR.getCaPlayer() == null) {
            return;
        }

        for (int i = 0; i < keyMapPressing.size(); i++) {
            int keyCode = keyMapPressing.keyAt(i);
            CvrSDK.getInstance().handleKeyCodeDown(keyCode);
        }
    }

    public static class KeyItemInfo {
        public int keycode;
        public String label;

        public KeyItemInfo(int keycode, String label) {
            this.keycode = keycode;
            this.label = label;
        }
    }
}
